/*****
 * Backwards.c
 *
 *	SoundEditor: Copyright (c) 1993 by
 *
 *	Motion Works Corp.
 *	#130 - 1020 Mainland Street
 *	Vancouver, BC
 *	(604) 685-9975
 *
 *  Author:	DMS		(13SEP93)
 *
 *	backwards sound effect (SE) component source for Marvin's Modulator
 *
 *****/

#include <MacHeaders>
#include <QuickTimeComponents.h>
#include <Sound.h>
#include <SoundInput.h>

	// prototype

pascal ComponentResult _SEWorksOnSelectionOnly(	Boolean *answer );

pascal ComponentResult _SEDoEffect( Handle theSound,
									Size *selectionStart, Size *selectionEnd,
									Handle *newSound,
									Size *insertStart, Size *deleteEnd );

static SoundHeaderPtr _GetTheSoundHeader( Handle theSound );


/***
 * _SEWorksOnSelectionOnly
 *
 *		returns TRUE to signify that it should only be enabled when there is a selection
 *
 ***/

pascal ComponentResult _SEWorksOnSelectionOnly(	Boolean *answer )
{
	*answer = TRUE;
	
	return noErr;
}



/***
 * _SEEffect
 *
 *		Reverse the selected samples and return a new sound with those samples in it.
 *
 ***/

pascal ComponentResult _SEDoEffect(	Handle theSound,
									Size *selectionStart, Size *selectionEnd,
									Handle *newSound,
									Size *insertStart, Size *deleteEnd )
{
	ComponentResult 		result = noErr;
	char					theSoundState;
	register Size			totalSamples;
	register Size			i;
	ExtSoundHeaderPtr		theExtHeaderPtr;
	SoundHeaderPtr 			theSoundHeaderPtr;
	char 					numChannels;

	*newSound = NULL;
	
	theSoundState = HGetState( theSound );
	HLock( theSound );
	theSoundHeaderPtr = _GetTheSoundHeader( theSound );
	
	if ( theSoundHeaderPtr->encode == stdSH )			/* sji */
	{
		register Ptr	from, to;
		SoundHeaderPtr	theNewSoundHeaderPtr;
		short			actualHeaderSize;
	
		totalSamples = *selectionEnd - *selectionStart;
		*newSound = NewHandle( 100 );	// big enough to hold largest possible sound header
		if ( *newSound )
		{
			SetupSndHeader( *newSound, 1, theSoundHeaderPtr->sampleRate, 8,
							(OSType)'NONE',  theSoundHeaderPtr->baseFrequency,
							0, &actualHeaderSize );
			SetHandleSize( *newSound, (long)(actualHeaderSize + totalSamples) );
			if ( MemError() != noErr )
			{
				DisposeHandle( *newSound );
				HUnlock( theSound );
				return MemError();
			}

			HLock( *newSound );
			theNewSoundHeaderPtr = _GetTheSoundHeader( *newSound );
			theNewSoundHeaderPtr->length = totalSamples;
			to = (Ptr)&theNewSoundHeaderPtr->sampleArea;
			from = theSoundHeaderPtr->sampleArea + *selectionEnd;
			
			for ( i = 0; i < totalSamples; i++ )
				*to++ = *from--;

			HUnlock( *newSound );
		}
	}
	else if ( theSoundHeaderPtr->encode == extSH )	/* sji */
	{
		short				numBytesPer;
		ExtSoundHeaderPtr	theNewSoundHeaderPtr;
		short				actualHeaderSize;
	
		theExtHeaderPtr = (ExtSoundHeaderPtr)theSoundHeaderPtr;
		numChannels = theExtHeaderPtr->numChannels;
		numBytesPer = numChannels * ( theExtHeaderPtr->sampleSize >> 3 );
		totalSamples = *selectionEnd - *selectionStart;	// in samples not bytes

		*newSound = NewHandle( 100 );	// big enough to hold largest possible sound header
		if ( *newSound )
		{
			SetupSndHeader( *newSound, theExtHeaderPtr->numChannels,
							theExtHeaderPtr->sampleRate, theExtHeaderPtr->sampleSize,
							(OSType)'NONE', theExtHeaderPtr->baseFrequency,
							0, &actualHeaderSize );
			SetHandleSize( *newSound, (long)(actualHeaderSize + totalSamples * numBytesPer) );
			if ( MemError() != noErr )
			{
				DisposeHandle( *newSound );
				HUnlock( theSound );
				return MemError();
			}

			HLock( *newSound );
			theNewSoundHeaderPtr = (ExtSoundHeaderPtr)_GetTheSoundHeader( *newSound );
			theNewSoundHeaderPtr->numFrames = totalSamples;

			switch ( numBytesPer )
			{
				case 1:
				{
					register char *from = theExtHeaderPtr->sampleArea + *selectionEnd;
					register char *to = (char *)&theNewSoundHeaderPtr->sampleArea;

					for ( i = 0; i < totalSamples; i++ )
						*to++ = *from--;
				}
					break;
		
				case 2:
				{
					register short *from = (short *)(theExtHeaderPtr->sampleArea + *selectionEnd * 2);
					register short *to = (short *)(&theNewSoundHeaderPtr->sampleArea);

					for ( i = 0; i < totalSamples; i++ )
						*to++ = *from--;
				}
					break;
				
				case 4:
				{
					register long *from = (long *)(theExtHeaderPtr->sampleArea + *selectionEnd * 4);
					register long *to = (long *)(&theNewSoundHeaderPtr->sampleArea);
	
					for ( i = 0; i < totalSamples; i++ )
						*to++ = *from--;
				}
					break;
					
			}

			HUnlock( *newSound );
		}
	}

	HSetState( theSound, theSoundState );

	if ( *newSound == NULL )
		result = MemError();
	else
	{
			// delete the old selection (thus replacing it with newSound)
		*insertStart = *selectionStart;
		*deleteEnd = *selectionEnd;
	}
	
	return result;
}


/**
 * _GetSoundHeader
 *
 *	Return a pointer to the SoundHeader. The sound handle must be locked
 *	while the pointer is in use.
 *
 **/

static SoundHeaderPtr _GetTheSoundHeader( Handle theSound )
{
	Ptr		sndData = *theSound;							// it's locked by now
	Ptr		cmd;
	short	numSynth;	// should be only one
	short	numCmds;	// should be only one
	short	cmdType;

	if ( *((short*)sndData) == 1 )							// format 1
	{
		numSynth = *((short*)(sndData + 2));
		numCmds = *((short*)(sndData + numSynth * 6 + 4));	// 6 bytes per synth + 4 others
		cmd = sndData + numSynth * 6 + 4 + 2;	
	}
	else if ( *sndData == 2 )								// format 2
	{
		numCmds = *((short *)(sndData + 4));				// 4 others
		cmd = sndData + 6;	
	}
	else
		return NULL;

	cmdType = *((short *)cmd) & 0x7FFF;
	if ( cmdType != bufferCmd && cmdType != soundCmd )
		return NULL;										// buffer cmd only

	return (SoundHeaderPtr)(sndData + *((long*)(cmd + 4)));	// param 2 == offset to data
}
