/*****
 * DoDialog.c
 *
 *	SoundEditor: Copyright (c) 1993 by
 *
 *	Motion Works Corp.
 *	#130 - 1020 Mainland Street
 *	Vancouver, BC
 *	(604) 685-9975
 *
 *  Author:	DMS		(16SEP93)
 *
 *	DoDialog routines for echo sound effect (SE) component source for Marvin's Modulator
 *
 *****/

#include <MacHeaders>
#include <Traps.h>

#define TABKEY			(0x09)
#define RETURNKEY		(0x0D)
#define ENTERKEY		(0x03)
#define	BSKEY			(0x08)
#define L_ARROWKEY		(0x1C)
#define R_ARROWKEY		(0x1D)
#define U_ARROWKEY		(0x1E)
#define D_ARROWKEY		(0x1F)
#define	HiliteMode		(Ptr)0x0938
#define	MIN(a,b)		(((a)<(b))?(a):(b))

	// prototype

Boolean DoDialog( short *delay, short *strength );


	// resource ID of dialog
#define	kDialogID		128

	// dialog items
enum
{
	kOKButton = 1,
	kCancelButton,
	kButtonUserItem,
	kDelayTextItem,
	kStrengthTextItem,
	kUnderlineUserItem
};

#define kFlashDelay		8			// ticks to delay while flashing button

	// glue

pascal OSErr SetDialogTrackCursor( DialogPtr theDialog, Boolean tracks )
				= {0x303C,0x0306,0xAA68};

pascal OSErr GetStdFilterProc( ProcPtr *theProc )
				= {0x303C, 0x0203, 0xAA68};




/***
 * GetDefault
 *
 ***/

static short GetDefault()
{
	return kOKButton;
}



/***
 * OutlineDefault
 *
 ***/

static pascal void OutlineDefault( DialogPtr theDialog, short theItem )
{
    short	defaultItem, notDefaultItem, itemType, diameter;
    Handle	itemHandle;
    Rect	itemRect;
	short	pnMode;

	notDefaultItem = ( ( defaultItem = GetDefault() ) != kOKButton
							? kOKButton : kCancelButton );
	SetPort( theDialog );
	
    GetDItem( theDialog, defaultItem, &itemType, &itemHandle, &itemRect );
    diameter = (itemRect.bottom - itemRect.top) / 2 + 6;
    if ( diameter < 16 )
        diameter = 16;
    PenSize( 3, 3 );
    InsetRect( &itemRect, -4, -4 );
    FrameRoundRect( &itemRect, diameter, diameter );
	
    GetDItem( theDialog, notDefaultItem, &itemType, &itemHandle, &itemRect );
    diameter = (itemRect.bottom - itemRect.top) / 2 + 6;
    if ( diameter < 16 )
        diameter = 16;
    PenSize( 3, 3 );
    pnMode = theDialog->pnMode;
    PenMode( patBic );
    InsetRect( &itemRect, -4, -4 );
    FrameRoundRect( &itemRect, diameter, diameter );
    PenMode( pnMode );
}



/***
 * DrawUnderline
 *
 ***/

static pascal void DrawUnderline( DialogPtr theDialog, short theItem )
{
    short	itemType;
    Handle	itemHandle;
    Rect	itemRect;
	
    GetDItem( theDialog, theItem, &itemType, &itemHandle, &itemRect );

	SetPort( theDialog );
	MoveTo( itemRect.left, itemRect.top );
	LineTo( itemRect.right, itemRect.top );
	itemRect.top += 2;
	MoveTo( itemRect.left, itemRect.top );
	LineTo( itemRect.right, itemRect.top );
}



/***
 * SetUpDialog
 *
 ***/

static DialogPtr SetUpDialog()
{
	DialogPtr	dialog;
	short		itemType;
	Handle		itemHandle;
	Rect		itemRect;

	dialog = GetNewDialog( kDialogID, NULL, (WindowPtr)-1L );

		// attach OutlineDefault to the user item
	GetDItem( dialog, kButtonUserItem, &itemType, &itemHandle, &itemRect );
	SetDItem( dialog, kButtonUserItem, itemType, (Handle)OutlineDefault, &itemRect );

		// attach DrawUnderline to its user item
	GetDItem( dialog, kUnderlineUserItem, &itemType, &itemHandle, &itemRect );
	SetDItem( dialog, kUnderlineUserItem, itemType, (Handle)DrawUnderline, &itemRect );

		// defaults
	GetDItem( dialog, kDelayTextItem, &itemType, &itemHandle, &itemRect );
	SetIText( itemHandle, "\p100" );

	GetDItem( dialog, kStrengthTextItem, &itemType, &itemHandle, &itemRect );
	SetIText( itemHandle, "\p60" );

	SetDialogTrackCursor( dialog, TRUE );

	InitCursor();

	ShowWindow( dialog );
	
	return dialog;
}



/***
 * GetItemHandle
 *
 *	from dlogUtils.c
 *
 ***/

/* ---------------------------------------------------------------------- */
static Handle	GetItemHandle(DialogPtr dp, short item)
	{
	short	iType;
	Handle	iHand;
	Rect	iRect;

	GetDItem(dp, item, &iType, &iHand, &iRect);
	return(iHand);
	}


/***
 * GetItemRect
 *
 *	from dlogUtils.c
 *
 ***/

/* ---------------------------------------------------------------------- */
static void	GetItemRect(DialogPtr dp, short item, Rect *theRect)
	{
	short	iType;
	Handle	iHand;

	GetDItem(dp, item, &iType, &iHand, theRect);
	}



/***
 * getEditText
 *
 *	from dlogUtils.c
 *
 ***/

/* ---------------------------------------------------------------------- */
static void	getEditText(DialogPtr theDlg, short theItem, char *returnStr, short maxChars)
	{
	char	pString[255];

	GetIText(GetItemHandle(theDlg, theItem), (StringPtr)pString);
	BlockMove(pString,returnStr,maxChars);
	returnStr[0] = (maxChars-1 < returnStr[0]) ? (maxChars-1) : returnStr[0];	/* truncate return string */
	}



/***
 * FlashEdit
 *
 *	from dlogUtils.c
 *
 ***/

/* ---------------------------------------------------------------------- */
static void	FlashEdit(DialogPtr dp)
	{
	Rect			theRect;
	GrafPtr			savedPort;
	long			timer;

	GetPort(&savedPort);
	SetPort(dp);

	GetItemRect(dp, ((DialogPeek)dp)->editField+1, &theRect);
	BitClr(HiliteMode,hiliteBit);
	InvertRect(&theRect);
	timer = TickCount() + 4;
	while (timer >= TickCount());
	BitClr(HiliteMode,hiliteBit);
	InvertRect(&theRect);
	
	SetPort(savedPort);
	}



/***
 * LimitNum
 *
 *	from stringUtils.c
 *
 ***/

/* ---------------------------------------------------------------------- */
static Boolean	isDigit(char ch)
	{
	return((ch >= '0' && ch <= '9') || ch == '-'); 
	}



/***
 * LimitNum
 *
 *	from dlogUtils.c
 *
 ***/

/* ---------------------------------------------------------------------- */
/* General purpose digit filtering routine; handles digits & editing only */
/* with length of input limited by parameter.  The maxlen parameter must  */
/* be between 0 and 39.                                                   */
/* ---------------------------------------------------------------------- */
static Boolean	LimitNum(DialogPtr dp, EventRecord *event, short maxlen)		/* most characters allowed */
	{
	TEHandle	hText;
	short	selBeg, selEnd;		/* selection beginning and end */
	short	textLen;			/* length of text to get */
	char	text[40];				/* for pulling the typed stuff */
	char	ch;

	if (event->what == keyDown || event->what == autoKey)
		{
		ch = event->message & charCodeMask;
		
		/* ignore input if return, enter, or cmd-. */
		if (ch == RETURNKEY || ch == ENTERKEY || ch == '.' && event->modifiers & cmdKey)
			return(true);

		textLen = MIN(maxlen+1, sizeof(text));	/* bound maxlen */

		/* other key; get text from current item and relevant TE handle */
		hText = ((DialogPeek)dp)->textH;
		textLen = MIN((*hText)->teLength+1, textLen);	/* limit length */
		getEditText(dp, ((DialogPeek)dp)->editField+1, text, textLen);
		/* get selection; if equal, insertion point => typing will add to length */
		selBeg = (*hText)->selStart;
		selEnd = (*hText)->selEnd;

		/* check for length: if = maxlen then char key will extend past limit
		 * unless key pressed was tab, backspace, or arrow keys
		 * unacceptable keys: all except digit and editing keys
		 * logic: if not a backspace and not a tab and not right/left arrow,
		 * and if either the char is illegal or typing will exceed max length
		 */
		if (ch != BSKEY && ch != TABKEY
			&& ch != L_ARROWKEY && ch != R_ARROWKEY
			&& ch != U_ARROWKEY && ch != D_ARROWKEY
			&& (!isDigit(ch) || (text[0] >= maxlen && selBeg == selEnd)))
			{
			FlashEdit(dp);
			event->what = nullEvent;
			return(true);
			}
		}
	return(false);	/* let ModalDialog handle */
	}



/***
 * FlashButton
 *
 ***/

static void FlashButton( DialogPtr dialog, short which )
{
	short			iType;
	ControlHandle	control;
	Rect			iRect;
	long			ticks;

	GetDItem( dialog, which, &iType, (Handle*)&control, &iRect );
	HiliteControl( control, inButton );
	Delay( kFlashDelay, &ticks );
	HiliteControl( control, 0 );
}



/***
 * DialogFilter
 *
 ***/

static pascal Boolean DialogFilter( DialogPtr theDialog, EventRecord *theEvent, short *itemHit )
{
	Boolean handled = FALSE;

	if ( theEvent->what == keyDown )	// just looking for keystrokes
	{
		char keypress = ( theEvent->message ) & charCodeMask;

		switch ( keypress )
		{
				// return or enter
			case 0x0d:
			case 0x03:
				FlashButton( theDialog, *itemHit = kOKButton );
				handled = TRUE;
				break;
				
				// ESC
			case 0x1b:
				FlashButton( theDialog, *itemHit = kCancelButton );
				handled = TRUE;
				break;
				
			case '.':
				if ( ( theEvent->modifiers ) & cmdKey )
				{
					FlashButton( theDialog, *itemHit = kCancelButton );
					handled = TRUE;
				}
				break;
				
			default:
				handled = LimitNum( theDialog, theEvent, 5 );
				break;
		}
	}
	else
	{
		ProcPtr standardProc;
		
		SetPort( theDialog );
		
		if ( ! GetStdFilterProc( &standardProc ) )
		{
			handled = ((ModalFilterProcPtr)standardProc)
							( theDialog, theEvent, itemHit );
		}
	}
	

	return handled;
}



/***
 * DoDialog
 *
 ***/

Boolean DoDialog( short *delay, short *strength )
{
	Boolean		ok = FALSE, done = FALSE;
	DialogPtr	dialog = SetUpDialog();
	short		itemHit = 0, itemType;
	Handle		itemHandle;
	Rect		itemRect;
	Str255		string;
	long		aLong;
	CursHandle	watchCursorHandle;
	
//	TESetJust( teFlushRight, ((DialogPeek)dialog)->textH );

	while ( !done )
	{
		ModalDialog( DialogFilter, &itemHit );
		{
			switch ( itemHit )
			{
				case kOKButton:
					ok = done = TRUE;
					break;

				case kCancelButton:
					ok = FALSE;
					done = TRUE;
					break;
			}
		}
	}
	
	if ( ok )
	{
		watchCursorHandle = GetCursor( watchCursor );
		SetCursor( *watchCursorHandle );

		GetDItem( dialog, kDelayTextItem, &itemType, &itemHandle, &itemRect );
		GetIText( itemHandle, string );
		StringToNum( string, &aLong );
		*delay = aLong;

		GetDItem( dialog, kStrengthTextItem, &itemType, &itemHandle, &itemRect );
		GetIText( itemHandle, string );
		StringToNum( string, &aLong );
		*strength = aLong;
		
		if ( ( !*delay ) || ( !*strength ) )
			ok = FALSE;
	}

	DisposDialog( dialog );
	
	return ok;
}
